import sys
import time
import glob
import os
import numpy as np
import tecplot as tp
from tecplot.constant import ExtractMode, PlotType
from collections import namedtuple 

if '-c' in sys.argv:
    tp.session.connect()

start = time.time()
tp.new_layout()

# Hide the Aspect Ratio warning that would otherwise come up
tp.macro.execute_command("$!Interface EnableWarnings = False")

files = glob.glob(os.path.join("Data","post*.h5"))
ds = tp.data.load_converge_hdf5(files)

# Setup some default style 
ds = tp.active_frame().dataset
frame = tp.active_frame()
plot = frame.plot()
plot.view.psi=0
plot.view.theta=0
plot.view.alpha=0
plot.show_edge=True
plot.show_shade=False
plot.view.center()

# Turn on the iso-surface and set some style so we can see it
plot.contour(0).variable = ds.variable("TEMPERATURE")
plot.show_isosurfaces=True
isosurface = plot.isosurface(0)
isosurface.isosurface_values[0] = 1800
isosurface.mesh.show=True
isosurface.mesh.color=plot.contour(0)
isosurface.mesh.line_thickness = 0.4
isosurface.contour.show = False

def get_zone_area(zone):
    """
    Use CFD Analyzer to compute the Length/Area/Volume of the supplied zone. CFD Analyzer stores the result in
    frame auxiliary data as a string and we extract it and convert to a float.
    """
    tp.macro.execute_extended_command(command_processor_id='CFDAnalyzer4',
        command="Integrate [{z}] VariableOption='LengthAreaVolume' IntegrateOver='Cells' IntegrateBy='Zones' PlotResults='F'".format(z=zone.index+1))
    return float(tp.active_frame().aux_data['CFDA.INTEGRATION_TOTAL'])

def get_max_x(zone):
    """
    Finds the Max X value and the corresponding Y-Value at that location. If there are multiple
    Max X-values, the lowest numbered node index is used.
    Returns a tuple containing the Max X value, the corresponding Y-value, and the 1-based node number associated with those values
    """
    xvals = zone.values('X')[:]
    yvals = zone.values('Y')[:]
    index = np.argmax(xvals)
    maxx = xvals[index]
    yval = yvals[index]
    # Return a 1-based node index because that's what Tecplot uses in the GUI
    return(maxx, yval, index+1)

with tp.session.suspend():
    # Create a namedtuple in which we store results. The tuple names are used as the variable
    # names in the resulting plot. Namedtuple does not allow spaces in names.
    Result = namedtuple("Result", ['Time', 'Flame_Area', 'MaxX', 'Y', 'MaxXIndex', 'Flame_Speed'])
    
    #
    # Loop over solution times.  At each timestep we extract the iso-surface which defines the flame
    # front and from that extracted zone we compute the area, max-x, etc.  Those values are stored
    # in a results list, where each element in the list is a 'Result' namedtuple
    #
    results = []
    for t in ds.solution_times:
        now = time.time()
        plot.solution_time = t
        iso_zone = isosurface.extract(mode=ExtractMode.SingleZone)
        area = get_zone_area(iso_zone)
        max_x, yval, index = get_max_x(iso_zone)
        
        # Compute the flame speed dx/dt
        speed = 0
        if results:
            last_result = results[-1]
            speed = (max_x - getattr(last_result,'MaxX')) / (t - getattr(last_result, 'Time'))
       
        result = Result(t, area, max_x, yval, index, speed)
        results.append(result)
        
        # Don't bother doing this work in batch mode
        if tp.session.connected:
            # Add Aux Data to the extracted iso-surface so we have the results stored there too.
            for varname in result._fields:
                iso_zone.aux_data[varname] = getattr(result,varname)
        print("Processed t={} in {} seconds".format(t, time.time()-now))

#
# Now that we have the results we'll add a new frame and create a new dataset to display the results
#
result_frame = tp.active_page().add_frame()
result_dataset = result_frame.create_dataset("Flame Front Analysis", var_names=Result._fields)
zone = result_dataset.add_ordered_zone("Flame Front Analysis", (len(results),1,1))
# Loop over the names in the named tuple and add the results for each variable
for varname in Result._fields:
    # Python magic to get the values for a specific variable from the results
    var_values = [getattr(n,varname) for n in results]
    zone.values(varname)[:] = var_values

# Rename the variables to have spaces so they look better (remember, the named tuple can't support spaces) 
for v in result_dataset.variables():
    v.name = v.name.replace("_", " ")

# Switch the frame to XY plot type to display the results
result_frame.plot_type = PlotType.XYLine
result_frame.activate()

print("Saving results")
tp.data.save_tecplot_ascii('results.dat', frame=result_frame, use_point_format=True)
tp.data.save_tecplot_plt('results.plt', frame=result_frame)

print("Creating layout file")
# The extraction of iso-surfaces "dirties" the data, so we'll start over to create a final plot
tp.new_layout()
ds = tp.data.load_converge_hdf5(files)
frame1 = tp.active_frame()
frame1.load_stylesheet("3d_isometric_view.sty", frame_geom=True)
frame2 = tp.active_page().add_frame()
frame2.load_stylesheet("3d_xy_plane_view.sty", frame_geom=True)
frame3 = tp.active_page().add_frame()
tp.data.load_tecplot("results.plt", frame=frame3)
frame3.load_stylesheet("xy_plot.sty", frame_geom=True)
tp.macro.execute_command("$!WorkspaceView FitAllFrames")
frame1.activate() # Make a 3D frame active so we get the time controls when we open the layout

# Modifications to colormaps are not stored in stylesheets, so we modify it here.
# The modification will be retained in the layout file.
tp.macro.execute_command("$!ColorMapAttributes 'cmocean - thermal' ControlPoint 1 {LeadRGB{R = 255}}")
tp.macro.execute_command("$!ColorMapAttributes 'cmocean - thermal' ControlPoint 1 {LeadRGB{G = 255}}")
tp.macro.execute_command("$!ColorMapAttributes 'cmocean - thermal' ControlPoint 1 {LeadRGB{B = 255}}")
tp.macro.execute_command("$!ColorMapAttributes 'cmocean - thermal' ControlPoint 1 {TrailRGB{R = 255}}")
tp.macro.execute_command("$!ColorMapAttributes 'cmocean - thermal' ControlPoint 1 {TrailRGB{G = 255}}")
tp.macro.execute_command("$!ColorMapAttributes 'cmocean - thermal' ControlPoint 1 {TrailRGB{B = 255}}")

tp.save_layout("result.lay")

print("Elapsed Time: ", time.time()-start)
