import sys
import time
import glob
import os
import numpy as np
import atexit, os, multiprocessing
import tecplot as tp
from tecplot.constant import ExtractMode, PlotType
from operator import attrgetter
from collections import namedtuple 


Result = namedtuple("Result", ['Time', 'Flame_Area', 'MaxX', 'Y', 'MaxXIndex', 'Flame_Speed'])

def init():
    atexit.register(tp.session.stop)
    tp.new_layout()

def get_zone_area(zone):
    """
    Use CFD Analyzer to compute the Length/Area/Volume of the supplied zone. CFD Analyzer stores the result in
    frame auxiliary data as a string and we extract it and convert to a float.
    """
    tp.macro.execute_extended_command(command_processor_id='CFDAnalyzer4',
        command="Integrate [{z}] VariableOption='LengthAreaVolume' IntegrateOver='Cells' IntegrateBy='Zones' PlotResults='F'".format(z=zone.index+1))
    return float(tp.active_frame().aux_data['CFDA.INTEGRATION_TOTAL'])

def get_max_x(zone):
    """
    Finds the Max X value and the corresponding Y-Value at that location. If there are multiple
    Max X-values, the lowest numbered node index is used.
    """
    xvals = zone.values('X')[:]
    yvals = zone.values('Y')[:]
    index = np.argmax(xvals)
    maxx = xvals[index]
    yval = yvals[index]
    # Return a 1-based node index because that's what Tecplot uses in the GUI
    return(maxx, yval, index+1)

def work(file):
    tp.new_layout()
    # Hide the Aspect Ratio warning that would otherwise come up
    tp.macro.execute_command("$!Interface EnableWarnings = False")

    try: 
        ds = tp.data.load_converge_hdf5(file)
    
        # Setup some default style 
        ds = tp.active_frame().dataset
        frame = tp.active_frame()
        plot = frame.plot()
        plot.view.psi=0
        plot.view.theta=0
        plot.view.alpha=0
        plot.show_edge=True
        plot.show_shade=False
        plot.view.center()
        
        # Turn on the iso-surface and set some style so we can see it
        plot.contour(0).variable = ds.variable("TEMPERATURE")
        plot.show_isosurfaces=True
        isosurface = plot.isosurface(0)
        isosurface.isosurface_values[0] = 1800
        isosurface.mesh.show=True
        isosurface.mesh.color=plot.contour(0)
        isosurface.mesh.line_thickness = 0.4
        isosurface.contour.show = False
    
        iso_zone = isosurface.extract(mode=ExtractMode.SingleZone)
        area = get_zone_area(iso_zone)
        max_x, yval, index = get_max_x(iso_zone)
        result = Result(ds.zone(0).solution_time, area, max_x, yval, index, 0)
        print(result)
    except:
        print("Error processing: ", file)
    return result

if __name__ == '__main__':
    now = time.time()
    multiprocessing.set_start_method('spawn')
    files = glob.glob(os.path.join("Data", "post*.h5"))

    num_workers = int(sys.argv[1]) #min(multiprocessing.cpu_count(), len(files))

    #manager = multiprocessing.Manager()
    #return_dict = manager.dict()
    pool = multiprocessing.Pool(num_workers, initializer=init)

    try:
        data = pool.map(work, files)
    finally:
        pool.close()
        pool.join()
        sorted_data = sorted(data, key=attrgetter('Time'))
        for i, item in enumerate(sorted_data):
            if i == 0:
                continue
            previous_item = sorted_data[i-1]
            max_x = getattr(item, 'MaxX')
            solution_time = getattr(item, 'Time')
            max_x_m1 = getattr(previous_item, 'MaxX')
            solution_time_m1 = getattr(previous_item, 'Time')
            speed = (max_x - max_x_m1) / (solution_time - solution_time_m1)
            sorted_data[i] = item._replace(Flame_Speed=speed)

        #
        # Now that we have the results create a new dataset and save the results to a file
        #
        tp.new_layout()
        result_frame = tp.active_page().active_frame()
        result_dataset = result_frame.create_dataset("Flame Front Analysis", var_names=Result._fields)
        zone = result_dataset.add_ordered_zone("Flame Front Analysis", (len(sorted_data),1,1))
        # Loop over the names in the named tuple and add the results for each variable
        for varname in Result._fields:
            # Python magic to get the values for a specific variable from the results
            var_values = [getattr(n,varname) for n in sorted_data]
            zone.values(varname)[:] = var_values
        
        # Switch the frame to XY plot type to display the results
        result_frame.plot_type = PlotType.XYLine
        result_frame.activate()
        
        print("Saving results")
        tp.data.save_tecplot_ascii('results.dat', frame=result_frame, use_point_format=True)
        tp.data.save_tecplot_plt('results.plt', frame=result_frame)
        
        print("Elapsed: ", time.time()-now)
